unit UTInfoTrackTh;

//=========================================================================
//
// I N T E R F A C E
//
//=========================================================================
interface

{$I JEDI.INC}

uses
  Classes;
type

  // Almacena informacin de una cancin.
  TInfoTrack = record
    ID:Integer;
    ArtistName:string;
    CollectionName:string;
    TrackName:string;
    Imagen100:string;
    Genero:string;
    Anyo:Integer;
    PathImage:string;
  end;

  TInfoTrackArray = array of TInfoTrack;

  //: Thread para obtener info. de una cancin
  TInfoTrackThread = class(TThread)
  private
    FArtista: string;
    FTitulo: string;
    FResponse: TStrings;
    FInfoArray: TInfoTrackArray;
    FNumResults: integer;
    FID: Integer;

    // Realiza el Parse de la resouesta
    procedure ParseResponseIndo(TS:TStrings);
    // descarga la cartula
    procedure DownloadImage(URLImage, ImageName:string; var imgPath:string);
  public
    // Interno
    property ID:Integer read FID write FID;
    // artista que utilizamos para la bsqueda
    property Artista:string read FArtista;
    // Ttulo que utilizamos para la bsqueda
    property Titulo:string read FTitulo;
    // Numero de resultados (mximo)
    property NumResults:integer read FNumResults write FNumResults;
    // La respuesta en formato JSON
    property Response:TStrings read FResponse write FResponse;
    // Array de resultados
    property InfoArray:TInfoTrackArray read FInfoArray write FInfoArray;
    // redefinir mtodos de la clase base
    procedure Execute; override;
    constructor Create(ID:Integer; AArtista, ATitulo:string);
    destructor Destroy; override;

  end;

//=========================================================================
//
// I M P L E M E N T A T I O N
//
//=========================================================================
implementation

uses
  {$IFDEF DELPHIXE5_UP}
  IdSSL, VCL.Dialogs,
  {$ENDIF}
  Forms, StrUtils,
  IdIOHandler, IdIOHandlerSocket, IdSSLOpenSSL, uLkJSON, SysUtils, idHTTP;



{ TInfoTrackThread }
constructor TInfoTrackThread.Create(ID:Integer; AArtista, ATitulo:string);
begin
  inherited Create(True);

  FID := ID;
  FArtista := AArtista;
  FTitulo := ATitulo;
  FNumResults := 5;    // default

  FResponse := TStringList.Create();
end;

destructor TInfoTrackThread.Destroy();
begin
  FreeAndNil(FResponse);
  inherited;
end;
  

//: recupara la imagen y la guarda en disco
procedure TInfoTrackThread.DownloadImage(URLImage, ImageName: string;
  var imgPath: string);
var
  Stream:TFileStream;
  IdH:TidHTTP;
  path:string;
begin
  // ini
  imgPath := '';
  // Nombre vaco
  if (ImageName = '') then begin
    Exit;
  end;
  // Path de salida
  path := IncludeTrailingBackslash(ExtractFilePath(Application.ExeName)) + 'temp\' + ImageName;
  // Crearlo por si no existe
  ForceDirectories(ExtractFilePath(path));
  try
    // Stream para la imagfen
    Stream  := TFileStream.Create(path, fmCreate);
    try
      //recuperar la imagen
      IdH := TidHTTP.Create(nil);
      IdH.AllowCookies := True;
      // proteccion
      try
        IdH.Get(Trim(URLImage), Stream);
      except
        // Error al descargar la imagen
        //..  Volcarlo al log
      end;
    finally
      // Liberar
      idH.Free;
      Stream.Free;
    end;
    // Path de salida
    imgPath := path;
  except
    // error al crear el fichero
    //...  Log
  end;
end;


procedure TInfoTrackThread.Execute();
const
  INI_URL = 'https://itunes.apple.com/search?term=%s&limit=%d';
var
  idHTTP:TidHTTP;
  s:String;
  ms:TMemoryStream;
  {$IFDEF DELPHIXE5_UP}
  IdSSL:TIdSSLIOHandlerSocketOpenSSL;
  {$ELSE}
  IdSSL:TIdSSLIOHandlerSocket;
  {$ENDIF}
begin

  inherited;

  // ini
  ms := TMemoryStream.Create;
  idHTTP := TidHTTP.Create(nil);
  FResponse.Clear;

  try
    // ini
    IdHTTP.ReadTimeout := 30000;
    IdSSL := {$IFDEF DELPHIXE5_UP}TIdSSLIOHandlerSocketOpenSSL{$ELSE}TIdSSLIOHandlerSocket{$ENDIF}.Create(nil);
    IdHTTP.IOHandler := IdSSL;
    IdSSL.SSLOptions.Method := sslvTLSv1;
    IdSSL.SSLOptions.Method := sslvTLSv1;
    IdSSL.SSLOptions.Mode := sslmUnassigned;
    idHTTP.HandleRedirects := True;

    // Montamos la cadena a solicitar
    s := AnsiReplaceText(FArtista, ' ', '+') +
         '+' +
         AnsiReplaceText(FTitulo, ' ', '+');
    s := Format(INI_URL, [s, FNumResults]);

    // Proteccion para solicitar datos
    try
      // Obtener resultado
      idHTTP.Get(s, ms);
      ms.Position:=0;
      FResponse.LoadFromStream(ms);
      // Realizar el Parse de la respuesta
      ParseResponseIndo(FResponse);


      // DEBUG!!!!!!!!!!!!!!!!!
      FResponse.SaveToFile('c:\temp\RESULT' + IntToStr(Self.FID) + '.txt');
    except
      on E:Exception do begin
        ShowMessage('Error al obtener informacin; ' + E.message);
      end;
    end;
  finally
    FreeAndNil(idHTTP);
    FreeAndNil(ms);
  end;
end;



procedure TInfoTrackThread.ParseResponseIndo(TS: TStrings);
var
  obj:TlkJSONobject;
  objData:TlkJSONobject;
  objRes:TlkJSONlist;
  i:integer;
  str:string;
begin
  // Conseguir la cadena formateada.
  obj := TlkJSONobject.Create();
  // proteccion
  try
    // Parse
  	TlkJSONBase(obj) := TlkJSON.ParseText(FResponse.Text);

    // Objeto results?
    if (obj.IndexOfName('results') <> -1) then begin
      // lista de resultados
      TlkJSONBase(objRes) := obj.Field['results'];

      // array de productos
      SetLength(FInfoArray, objRes.Count);

      // Recorrer la lista de productos.
      for i := 0 to (objRes.Count - 1) do begin
        // Datos del producto
        TlkJSONBase(objData) := objRes.Child[i];

        // Obtener datos
        FInfoArray[i].ID := Self.FID;
        FInfoArray[i].ArtistName := objData.Field['artistName'].Value;
        FInfoArray[i].CollectionName := objData.Field['collectionName'].Value;
        FInfoArray[i].TrackName := objData.Field['trackName'].Value;
        FInfoArray[i].Imagen100 := objData.Field['artworkUrl100'].Value;
        FInfoArray[i].Genero := objData.Field['primaryGenreName'].Value;
        Str := objData.Field['releaseDate'].Value;
        Str := Copy(Str, 1, 4);
        FInfoArray[i].Anyo := StrToIntDef(Str, 0);

        // descargar imagen
        DownloadImage(FInfoArray[i].Imagen100, IntToStr(Self.FID) + '.jpg', Str);
        FInfoArray[i].PathImage := str;
      end;
    end;

  finally
	  FreeAndNil(obj);
  end;

end;


end.
